=== modified file 'NEWS'
--- NEWS	2005-10-18 18:52:56 +0000
+++ NEWS	2005-10-19 02:40:58 +0000
@@ -1,8 +1,13 @@
 UNRELEASED CHANGES
 
   IMPROVEMENTS:
+    * merge now takes a --show-base option to include the base text in
+      conflicts.
+      (Aaron Bentley)
+
     * The config files are now read using ConfigObj, so '=' should be used as
       a separator, not ':'.
+      (Aaron Bentley)
 
     * The config directory is now ~/.bazaar, and there is a single file 
       ~/.bazaar/bazaar.conf storing email, editor and other preferences.

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-17 20:17:55 +0000
+++ bzrlib/builtins.py	2005-10-19 02:40:58 +0000
@@ -1269,10 +1269,12 @@
     --force is given.
     """
     takes_args = ['branch?']
-    takes_options = ['revision', 'force', 'merge-type']
+    takes_options = ['revision', 'force', 'merge-type', 
+                     Option('show-base', help="Show base revision text in "
+                            "conflicts")]
 
-    def run(self, branch=None, revision=None, force=False, 
-            merge_type=None):
+    def run(self, branch=None, revision=None, force=False, merge_type=None,
+            show_base=False):
         from bzrlib.merge import merge
         from bzrlib.merge_core import ApplyMerge3
         if merge_type is None:
@@ -1303,7 +1305,8 @@
                 other = [branch, revision[1].in_history(b).revno]
 
         try:
-            merge(other, base, check_clean=(not force), merge_type=merge_type)
+            merge(other, base, check_clean=(not force), merge_type=merge_type,
+                  show_base=show_base)
         except bzrlib.errors.AmbiguousBase, e:
             m = ("sorry, bzr can't determine the right merge base yet\n"
                  "candidates are:\n  "

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-10-17 05:18:20 +0000
+++ bzrlib/merge.py	2005-10-19 02:40:58 +0000
@@ -241,7 +241,7 @@
 def merge(other_revision, base_revision,
           check_clean=True, ignore_zero=False,
           this_dir=None, backup_files=False, merge_type=ApplyMerge3,
-          file_list=None):
+          file_list=None, show_base=False):
     """Merge changes into a tree.
 
     base_revision
@@ -324,7 +324,8 @@
                                           " tree." % fname)
         merge_inner(this_branch, other_tree, base_tree, tempdir, 
                     ignore_zero=ignore_zero, backup_files=backup_files, 
-                    merge_type=merge_type, interesting_ids=interesting_ids)
+                    merge_type=merge_type, interesting_ids=interesting_ids,
+                    show_base=show_base)
         if base_is_ancestor and other_rev_id is not None\
             and other_rev_id not in this_branch.revision_history():
             this_branch.add_pending_merge(other_rev_id)
@@ -342,10 +343,13 @@
 
 def merge_inner(this_branch, other_tree, base_tree, tempdir, 
                 ignore_zero=False, merge_type=ApplyMerge3, backup_files=False,
-                interesting_ids=None):
+                interesting_ids=None, show_base=False):
 
     def merge_factory(file_id, base, other):
-        contents_change = merge_type(file_id, base, other)
+        if show_base is True:
+            contents_change = merge_type(file_id, base, other, show_base=True)
+        else:
+            contents_change = merge_type(file_id, base, other)
         if backup_files:
             contents_change = BackupBeforeChange(contents_change)
         return contents_change

=== modified file 'bzrlib/merge3.py'
--- bzrlib/merge3.py	2005-09-04 02:59:56 +0000
+++ bzrlib/merge3.py	2005-10-19 02:40:58 +0000
@@ -76,16 +76,19 @@
     def merge_lines(self,
                     name_a=None,
                     name_b=None,
+                    name_base=None,
                     start_marker='<<<<<<<',
                     mid_marker='=======',
                     end_marker='>>>>>>>',
-                    show_base=False):
+                    base_marker=None):
         """Return merge in cvs-like form.
         """
         if name_a:
             start_marker = start_marker + ' ' + name_a
         if name_b:
             end_marker = end_marker + ' ' + name_b
+        if name_base and base_marker:
+            base_marker = base_marker + ' ' + name_base
             
         for t in self.merge_regions():
             what = t[0]
@@ -102,6 +105,10 @@
                 yield start_marker + '\n'
                 for i in range(t[3], t[4]):
                     yield self.a[i]
+                if base_marker is not None:
+                    yield base_marker + '\n'
+                    for i in range(t[1], t[2]):
+                        yield self.base[i]
                 yield mid_marker + '\n'
                 for i in range(t[5], t[6]):
                     yield self.b[i]

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-10-14 04:53:26 +0000
+++ bzrlib/merge_core.py	2005-10-19 02:40:58 +0000
@@ -9,10 +9,11 @@
 
 class ApplyMerge3:
     """Contents-change wrapper around merge3.Merge3"""
-    def __init__(self, file_id, base, other):
+    def __init__(self, file_id, base, other, show_base=False):
         self.file_id = file_id
         self.base = base
         self.other = other
+        self.show_base = show_base
 
     def is_creation(self):
         return False
@@ -49,8 +50,13 @@
         new_conflicts = False
         output_file = file(new_file, "wb")
         start_marker = "!START OF MERGE CONFLICT!" + "I HOPE THIS IS UNIQUE"
+        if self.show_base is True:
+            base_marker = '|' * 7
+        else:
+            base_marker = None
         for line in m3.merge_lines(name_a = "TREE", name_b = "MERGE-SOURCE", 
-                       start_marker=start_marker):
+                       name_base = "BASE-REVISION",
+                       start_marker=start_marker, base_marker=base_marker):
             if line.startswith(start_marker):
                 new_conflicts = True
                 output_file.write(line.replace(start_marker, '<' * 7))

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-10-17 20:17:55 +0000
+++ bzrlib/selftest/blackbox.py	2005-10-19 02:40:58 +0000
@@ -497,7 +497,19 @@
         file('question', 'wb').write("What do you get when you multiply six"
                                    "times nine?")
         self.runbzr('commit -m this')
+        self.runbzr('merge ../other --show-base')
+        conflict_text = file('hello').read()
+        assert '<<<<<<<' in conflict_text
+        assert '>>>>>>>' in conflict_text
+        assert '=======' in conflict_text
+        assert '|||||||' in conflict_text
+        assert 'hi world' in conflict_text
+        self.runbzr('revert')
+        self.runbzr('resolve --all')
         self.runbzr('merge ../other')
+        conflict_text = file('hello').read()
+        assert '|||||||' not in conflict_text
+        assert 'hi world' not in conflict_text
         result = self.runbzr('conflicts', backtick=1)
         self.assertEquals(result, "hello\nquestion\n")
         result = self.runbzr('status', backtick=1)

